<?php

defined('ABSPATH') or exit;

/**
 * Create application draft
 */
class Montonio_Financing_Request
{

    /**
     * @var integer
     */
    protected $publicKey;

    /**
     * @var string
     */
    protected $secretKey;

    /**
     * @var string
     */
    protected $returnUrl;

    /**
     * @var boolean
     */
    protected $sandbox;

    protected $draftData;

    protected $orderID;

    protected $order;

    /**
     * Required when Redirect Customer Journey
     * 
     * @var string
     */
    protected $notificationUrl;

    /**
    * Required when Redirect Customer Journey
    * 
    * @var string
    */
    protected $callbackUrl;

    /**
     * @return self
     */
    public static function create()
    {
        return new self();
    }

    /**
     * Montonio_Financing_Request constructor.
     */
    public function __construct()
    {}

    public function prepareCustomerDetails()
    {
        $draftData = array(
            "origin"              => "online",
            "merchant_reference"  => (string) $this->orderID,
            "customer_first_name" => (string) $this->order->get_billing_first_name(),
            "customer_last_name"  => (string) $this->order->get_billing_last_name(),
            "customer_email"      => (string) $this->order->get_billing_email(),
            "customer_phone"      => (string) $this->order->get_billing_phone(),

            "customer_city"        => (string) $this->order->get_billing_city(),
            "customer_address"     => (string) $this->order->get_billing_address_1(),
            "customer_postal_code" => (string) $this->order->get_billing_postcode(),

            "products"            => array(),
        );

        if ($this->getCallbackUrl()) {
            $draftData['callback_url'] = $this->getCallbackUrl();
        }

        if ($this->getNotificationUrl()) {
            $draftData['notification_url'] = $this->getNotificationUrl();
        }

        return $this->setDraftData($draftData);
    }

    public function prepareProducts()
    {

        $order     = $this->getOrder();
        $draftData = $this->getDraftData();

        foreach ($order->get_items() as $item_id => $item_data) {

            // Get an instance of corresponding the WC_Product object
            $product       = $item_data->get_product();
            $item_quantity = $item_data->get_quantity(); // Get the item quantity
            $product_name  = $product->get_name(); // Get the product name
            $product_price = $item_data->get_total(); // Get the product price
            $product_tax   = $item_data->get_total_tax() ? $item_data->get_total_tax() : 0; // Get the product taxes

            // add a product x quantity
            $draftData["products"][] = array(
                "product_name"  => $product_name,
                "product_price" => (float) round(($product_price + $product_tax) / $item_data->get_quantity(), 2),
                "quantity"      => (int) $item_quantity,
            );
        }

        // shipping cost
        $shipping_cost = $order->get_shipping_total();
        $shipping_tax  = $order->get_shipping_tax() ? $order->get_shipping_tax() : 0;
        if ($shipping_cost && $shipping_cost > 0) {
            $draftData["products"][] = array(
                "product_name"  => 'SHIPPING',
                "product_price" => (float) round($shipping_cost + $shipping_tax, 2),
                "quantity"      => 1,
            );
        }

        return $this->setDraftData($draftData);
    }

    /**
     * Create request url
     *
     * @param array $parameters
     *
     * @return string
     */
    public function buildUrl()
    {
        // create new API instance
        if (!class_exists('MontonioAPI')) {
            require_once 'libraries/MontonioAPI.php';
        }

        $api = new MontonioAPI(
            $this->getPublicKey(),
            $this->getSecretKey(),
            ($this->getSandbox()) ? 'sandbox' : 'production'
        );

        $draftData = $this->getDraftData();

        // get application draft
        $draft_result = $api->post_montonio_application_draft(json_encode($draftData));

        // handle draft request response
        $draft_token = ($draft_result['status'] === 'SUCCESS') ? $draft_result['data']->access_token : null;

        $url   = $this->getReturnUrl();
        $query = parse_url($url, PHP_URL_QUERY);

        if ($query) {
            $url .= '&access_token=' . $draft_token;
        } else {
            $url .= '?access_token=' . $draft_token;
        }

        return $url;
    }

    public function getDraftData()
    {
        return $this->draftData;
    }

    public function setDraftData($draftData)
    {
        $this->draftData = $draftData;
        return $this;
    }

    public function getOrder()
    {
        return $this->order;
    }

    public function setOrder($order)
    {
        $this->order = $order;
        return $this;
    }

    public function getOrderID()
    {
        return $this->orderID;
    }

    public function setOrderID($orderID)
    {
        $this->orderID = $orderID;
        return $this;
    }

    public function getReturnUrl()
    {
        return $this->returnUrl;
    }

    public function setReturnUrl($returnUrl)
    {
        $this->returnUrl = $returnUrl;
        return $this;
    }

    /**
     * @return integer
     */
    public function getPublicKey()
    {
        return $this->publicKey;
    }

    /**
     * @param integer $publicKey
     *
     * @return self
     */
    public function setPublicKey($publicKey)
    {
        $this->publicKey = $publicKey;
        return $this;
    }

    /**
     * @return string
     */
    public function getSecretKey()
    {
        return $this->secretKey;
    }

    /**
     * @param string $secretKey
     *
     * @return self
     */
    public function setSecretKey($secretKey)
    {
        $this->secretKey = $secretKey;
        return $this;
    }

    /**
     * @return boolean
     */
    public function getSandbox()
    {
        return $this->sandbox;
    }

    /**
     * @param boolean $sandbox
     *
     * @return self
     */
    public function setSandbox($sandbox)
    {
        $this->sandbox = $sandbox;
        return $this;
    }

    /**
    * @return boolean
    */
    public function getNotificationUrl()
    {
        return $this->notificationUrl;
    }

    /**
    * @param boolean $notificationUrl
    *
    * @return self
    */
    public function setNotificationUrl($notificationUrl)
    {
        $this->notificationUrl = $notificationUrl;
        return $this;
    }

    /**
    * @return boolean
    */
    public function getCallbackUrl()
    {
        return $this->callbackUrl;
    }

    /**
    * @param boolean $callbackUrl
    *
    * @return self
    */
    public function setCallbackUrl($callbackUrl)
    {
        $this->callbackUrl = $callbackUrl;
        return $this;
    }
}
