<?php

defined('ABSPATH') or exit;

if (!class_exists('HirepurchaseValidator')) {
    require_once 'libraries/HirepurchaseValidator.php';
}

/**
* Class for handling Montonio hire purchase application result
*/
class Montonio_Financing_Validation
{
    
    const SIGNED              = 'signed';
    const PENDING             = 'pending';
    const REJECTED            = 'rejected';
    const CANCELLED           = 'cancelled';
    const OFFERS_RECEIVED     = 'offers_received';
    const AWAITING_SIGNATURE  = 'awaiting_signature';
    const EXTRA_INFO_REQUIRED = 'extra_info_required';
    
    /**
    * @var string
    * Status for successful laon application
    */
    protected $paymentCompletedStatus;
    
    /**
    * @var string
    * Status if any errors occurred, cancel order
    */
    protected $paymentCancelledStatus;
    
    /**
    * @var integer
    */
    protected $publicKey;
    
    /**
    * @var string
    */
    protected $secretKey;
    
    /**
    * @var string
    */
    protected $returnUrl;
    
    /**
    * @var boolean
    */
    protected $sandbox;
    
    /**
    * @var string/int
    */
    protected $orderID;
    
    /**
    * @var object
    */
    protected $order;
    
    /**
    * @var object
    */
    protected $application;
    
    public function __construct() {
        
    }
    
    public static function create() {
        return new self();
    }
    
    public function getReturnUrlForApplication() {
        
        if (!class_exists('MontonioAPI')) {
            require_once 'libraries/MontonioAPI.php';
        }
        
        $order = $this->getOrder();
        
        // if not handled for any reason, redirect the user to homepage
        $return_url = wc_get_checkout_url();
        
        // create new API instance
        $api = new MontonioAPI(
            $this->getPublicKey(),
            $this->getSecretKey(),
            ($this->getSandbox()) ? 'sandbox' : 'production'
        );
        
        // get the application for this order from Montonio
        $res = $api->get_montonio_application( $this->getOrderID() );
        if ( $res['status'] == 'SUCCESS') $this->application = $res['data'];
        else {
            wc_add_notice('There was a problem with processing the order. ', 'error');
            $order->add_order_note(__('Montonio: Unable to process payment. Payment was not processed. Please contact support.', 'woo-montonio'));
            $this->updateOrderStatus($order, $this->getPaymentCancelledStatus());
            return $return_url;
        }
        
        // FRAUD DETECTION
        try {
            Montonio_Hirepurchase_Validator::checkMethod($order->get_payment_method());
            Montonio_Hirepurchase_Validator::checkId($this->application->merchant_reference, $this->getOrderID());
            Montonio_Hirepurchase_Validator::checkAmount($this->application->loan_amount, $order->get_total());
            Montonio_Hirepurchase_Validator::checkCurrency($this->application->currency_code, $order->get_currency());
        } catch(Exception $e) {
            $order->add_order_note(__('Montonio: Unable to verify integrity of hire purchase application. Payment was not processed. Please contact support.', 'woo-montonio'));
            $this->updateOrderStatus($order, $this->getPaymentCancelledStatus());
            wc_add_notice('Payment failed. Montonio application was not completed.', 'error');
            return $return_url;
        }
        
        switch ($this->application->status) {
            case self::SIGNED:  // Hire purchase agreement signed
            $order->add_order_note(__('Montonio: Callback order payment completed', 'woo-montonio'));
            $this->updateOrderStatus($order, $this->getPaymentCompletedStatus());
            return $order->get_checkout_order_received_url();
            
            default:
            $order->add_order_note(__('Montonio: Order validation failed', 'woo-montonio'));
            $this->updateOrderStatus($order, $this->getPaymentCancelledStatus());
            wc_add_notice('Payment failed. Montonio application was not completed.', 'error');
            return $return_url;
        }
    }
    
    protected function updateOrderStatus($order, $status)
    {
        $orderStatusFiltered = str_replace("wc-", "", $status);
        $order->update_status(
            $orderStatusFiltered,
            __('Montonio: Status changed to ', 'woo-montonio') . $orderStatusFiltered . '<br />',
            true
        );
    }
    
    
    
    // MARK: Getters and Setters ***********************************************
    
    public function getApplication() {
        return $this->application;
    }
    
    public function setApplication($application) {
        $this->application = $application;
        return self();
    }
    
    /**
    * @return string
    */
    public function getPublicKey()
    {
        return $this->publicKey;
    }
    
    /**
    * @param string $publicKey
    *
    * @return self
    */
    public function setPublicKey($publicKey)
    {
        $this->publicKey = $publicKey;
        return $this;
    }
    
    /**
    * @return string
    */
    public function getSecretKey()
    {
        return $this->secretKey;
    }
    
    /**
    * @param string $secretKey
    *
    * @return self
    */
    public function setSecretKey($secretKey)
    {
        $this->secretKey = $secretKey;
        return $this;
    }
    
    public function getReturnUrl()
    {
        return $this->returnUrl;
    }
    
    public function setReturnUrl($returnUrl)
    {
        $this->returnUrl = $returnUrl;
        return $this;
    }
    
    /**
    * @return boolean
    */
    public function getSandbox()
    {
        return $this->sandbox;
    }
    
    /**
    * @param boolean $sandbox
    *
    * @return self
    */
    public function setSandbox($sandbox)
    {
        $this->sandbox = $sandbox;
        return $this;
    }
    
    public function getOrder() {
        return $this->order;
    }
    
    public function setOrder($order) {
        $this->order = $order;
        return $this;
    }
    
    public function getOrderID() {
        return $this->orderID;
    }
    
    public function setOrderID($orderID) {
        $this->orderID = $orderID;
        return $this;
    }
    
    /**
    * @return string
    */
    public function getPaymentCompletedStatus()
    {
        return $this->paymentCompletedStatus;
    }
    
    /**
    * @param string $paymentCompletedStatus
    */
    public function setPaymentCompletedStatus($paymentCompletedStatus)
    {
        $this->paymentCompletedStatus = $paymentCompletedStatus;
        return $this;
    }
    
    /**
    * @return string
    */
    public function getPaymentCancelledStatus()
    {
        return $this->paymentCancelledStatus;
    }
    
    /**
    * @param string $paymentCancelledStatus
    */
    public function setPaymentCancelledStatus($paymentCancelledStatus)
    {
        $this->paymentCancelledStatus = $paymentCancelledStatus;
        return $this;
    }
}
?>
