<?php
defined('ABSPATH') or exit;

class Montonio_Financing extends WC_Payment_Gateway
{
    /**
     * Montonio image location
     */
    const MONTONIO_LOGO_URI = 'https://public.montonio.com/logo/montonio_dark.png';

    /**
     * Local language delimiter
     */
    const LANG_DELIMITER = '_';

    /**
     * Customer Journeys (prevent typos)
     */
    const CUSTOMER_JOURNEY_IFRAME   = 'iframe';
    const CUSTOMER_JOURNEY_REDIRECT = 'redirect';

    /**
     * Redirect Customer Journey URLs
     */
    const REDIRECT_JOURNEY_SANDBOX_URL = 'https://sandbox-application.montonio.com';
    const REDIRECT_JOURNEY_URL         = 'https://application.montonio.com';

    /**
     * Prevent typos
     */
    const NONCE_HIREPURCHASE_CALLBACK = 'montonio-hirepurchase-callback-';

    /**
     * @var string
     */
    protected $publicKey;

    /**
     * @var string
     */
    protected $secretKey;

    /**
     * @var boolean
     */
    protected $sandbox;

    /**
     * @var string
     */
    protected $paymentCancelledStatus;

    /**
     * @var string
     */
    protected $paymentCompletedStatus;

    /**
     * @var string
     */
    protected $paymentPendingStatus;

    /**
     * @var object
     */
    protected $pluginSettings;

    /**
     * @var string
     */
    protected $paymentHandleLogo;

    /**
     * @var bool
     */
    protected $showLogo;

    /**
     * @var bool
     */
    protected $showTitle;

    /**
     * @example 'iframe' | 'redirect'
     * @var string
     */
    protected $customerJourneyType;

    /**
     * WC_Gateway_Montonio constructor.
     */
    public function __construct()
    {
        if (!class_exists('MontonioAPI')) {
            require_once 'libraries/MontonioAPI.php';
        }

        $this->id                 = 'montonio';
        $this->has_fields         = true;
        $this->method_title       = __('Montonio Financing', 'woo-montonio');
        $this->method_description = __('Allows customers to apply for financing through Montonio.', 'woo-montonio');

        $this->init_form_fields();
        $this->init_settings();

        $this->showTitle = $this->get_option('showTitle');
        if ($this->showTitle !== 'no') {
            $this->title = $this->get_option('title');
        }

        $this->description = $this->get_option('description');
        $this->publicKey   = $this->get_option('publicKey');
        $this->secretKey   = $this->get_option('secretKey');
        $this->sandbox     = $this->get_option('sandbox') === 'yes';

        $this->showLogo          = $this->get_option('showPaymentHandleLogo');
        $this->paymentHandleLogo = $this->get_option('paymentHandleLogo');

        /**
         * Set the customer journey type
         */
        if (self::CUSTOMER_JOURNEY_IFRAME !== $this->get_option('customerJourneyType')) {
            $this->customerJourneyType = self::CUSTOMER_JOURNEY_REDIRECT;
        } else {
            $this->customerJourneyType = self::CUSTOMER_JOURNEY_IFRAME;
        }

        $this->paymentCancelledStatus = $this->get_option('paymentCancelledStatus');
        $this->paymentCompletedStatus = $this->get_option('paymentCompletedStatus');
        $this->paymentPendingStatus   = $this->get_option('paymentPendingStatus');

        // show logo if option enabled
        if ($this->getShowLogo() !== 'no') {
            if (strlen($this->getPaymentHandleLogo()) > 0) {
                $this->icon = $this->getPaymentHandleLogo();
            } else {
                $this->icon = $this::MONTONIO_LOGO_URI;
            }
        }

        if (self::CUSTOMER_JOURNEY_IFRAME === $this->customerJourneyType) {

            // This is where we init Montonio Gateway in iframe flow
            add_action('woocommerce_receipt_' . $this->id, array($this, 'setup_montonio_form'));

            // this is for callbacks
            add_action('woocommerce_api_montonio_update', array(&$this, 'update_merchant_system'));
        } else {

            /**
             * Register the callback function for when returning from Montonio Hirepurchase
             */
            add_action('woocommerce_api_montonio_hirepurchase_return', array($this, 'check_hirepurchase_response'));

            /**
             * Register payment notification webhook handler
             */
            add_action('woocommerce_api_montonio_hirepurchase_notification', array($this, 'check_hirepurchase_notification'));
        }

        /**
         * Admin options
         */
        add_action(
            'woocommerce_update_options_payment_gateways_montonio',
            array($this, 'process_admin_options')
        );
    }

    // =========================================================================
    // Admin Settings Page
    // =========================================================================

    public function init_form_fields()
    {
        if (!class_exists('Montonio_Financing_Settings')) {
            require_once 'class-montonio-financing-settings.php';
        }

        $this->setPluginSettings(Montonio_Financing_Settings::create());

        $this->form_fields = $this->getPluginSettings()->getFormFields();
    }

    public function admin_options()
    {
        $this->getPluginSettings()->setLang($this->getLocalLang($this::LANG_DELIMITER));
        $this->getPluginSettings()->setCurrency(get_woocommerce_currency());
        $this->getPluginSettings()->setPublicKey($this->getPublicKey());
        $this->updateAdminSettings($this->getPluginSettings()->generateNewSettings());
        $all_fields = $this->get_form_fields();

        // get form fields for Loan Calculator
        $calculatorOptionsCount = count($this->getPluginSettings()->getCalculatorFormFields());

        $tabs = $this->generateTabs(array(
            array(
                'name'  => __('Main Settings', 'woo-montonio'),
                'slice' => array_slice($all_fields, 0, count($all_fields) - $calculatorOptionsCount),
            ),
            array(
                'name'  => __('Calculator Widget Settings', 'woo-montonio'),
                'slice' => array_slice($all_fields, -$calculatorOptionsCount, count($all_fields)),
            ),
        ));

        $this->getPluginSettings()->buildAdminFormHtml($tabs);
    }

    // =========================================================================
    // Checkout page
    // =========================================================================

    /**
     * Resolve Payment handle style
     *
     * @return void
     */
    public function payment_fields()
    {
        $style = $this->_resolvePaymentHandleStyle($this->get_option('paymentHandleStyle'));

        wp_enqueue_style(
            'custom-frontend-style',
            MONTONIO_PLUGIN_URL . 'financing/assets/css/payment-handle/' . $style
        );

        if ($description = $this->getDescription()) {
            echo wpautop(wptexturize($description));
        }
    }

    /**
     * The method that executes after pressing "Place Order"
     *
     * @param int $order_id
     * @return array
     */
    public function process_payment($order_id)
    {
        error_log(
            'Order #' . $order_id . ' is redirected to payment.'
            . 'Notify URL: ' . trailingslashit(home_url()) . '?montonioListener=montonio_callback'
        );

        $order = wc_get_order($order_id);
        $order->add_order_note(__('Montonio: Order checkout process is started', 'woo-montonio'));
        $this->updateOrderStatus($order, $this->getPaymentPendingStatus());

        /**
         * Set the Return URL: Receipt page when Iframe, Redirect to Montonio otherwise
         * Also set Callback and Notification URLs when Redirect
         */
        if (self::CUSTOMER_JOURNEY_IFRAME === $this->customerJourneyType) {
            $draftRequest = $this->prepareIframeJourneyData($order_id, $order);
        } else {
            $draftRequest = $this->prepareRedirectJourneyData($order_id, $order);
        }

        /**
         * Reduce stock if not already reduced
         */
        wc_reduce_stock_levels($order_id);

        return array(
            'result'   => 'success',
            'redirect' => $draftRequest->buildUrl(),
        );
    }

    // =========================================================================
    // Redirect Customer Journey
    // =========================================================================

    /**
    * Prepare Redirect customer journey data for given order
    *
    * @param int $order_id
    * @param WC_Abstract_Order
    *
    * @return Montonio_Financing_Request
    *
    */
    protected function prepareRedirectJourneyData($order_id, $order)
    {
        if (!class_exists('Montonio_Financing_Request')) {
            require_once 'class-montonio-financing-request.php';
        }

        $redirectRequest = Montonio_Financing_Request::create()
            ->setPublicKey($this->getPublicKey())
            ->setSecretKey($this->getSecretKey())
            ->setSandbox($this->getSandbox())
            ->setOrderID($order_id)
            ->setOrder($order);

        /**
        * Set notification URL (Webhook after successfully signed hirepurchase agreement)
        */
        $notificationUrl = add_query_arg('wc-api', 'montonio_hirepurchase_notification', home_url());

        /**
        * Create callback URL and nonce it (when customer presses Return To Merchant)
        */
        $callbackUrl = add_query_arg(array(
            'wc-api'   => 'montonio_hirepurchase_return',
            'order_id' => $order_id,
        ), home_url());

        $noncedCallbackUrl = add_query_arg('_wpnonce', wp_create_nonce(self::NONCE_HIREPURCHASE_CALLBACK . $order_id), $callbackUrl);

        $redirectRequest
            ->setNotificationUrl($notificationUrl)
            ->setCallbackUrl($noncedCallbackUrl)
            ->setReturnUrl($redirectRequest->getSandbox() ? self::REDIRECT_JOURNEY_SANDBOX_URL : self::REDIRECT_JOURNEY_URL)
            ->prepareCustomerDetails()
            ->prepareProducts();

        return $redirectRequest;
    }

    /**
     * Handle notification coming from Montonio Hirepurchase
     * @requires $this->customerJourneyType === 'redirect'
     * @since 1.2.0
     *
     * @return void
     */
    public function check_hirepurchase_notification()
    {
        return $this->check_hirepurchase_response(true);
    }

    /**
     * Check callback from Montonio
     * and redirect user: thankyou page for success, checkout on declined/failure
     *
     * @param bool $is_notification True if webhook notification, false otherwise
     * @since 1.2.0
     *
     * @return void
     */
    public function check_hirepurchase_response($is_notification = false)
    {
        // New way of validating order: Signature verification and notification
        if ($is_notification) {

            $json = file_get_contents('php://input');

            /**
             * @var object
             */
            $body = json_decode($json);

            /**
             * @var array
             */
            $headers = getallheaders();

            $responseAccessKey = sanitize_text_field($headers['X-Montonio-Access-Key']);
            if ($responseAccessKey !== $this->getPublicKey()) {
                http_response_code(403);
                die();
            }

            if (!class_exists('MontonioAPI')) {
                require_once 'libraries/MontonioAPI.php';
            }

            $api = new MontonioAPI(
                $this->getPublicKey(),
                $this->getSecretKey(),
                ($this->getSandbox()) ? 'sandbox' : 'production'
            );

            $signature        = sanitize_text_field($headers['X-Montonio-Signature']);
            $signatureMatches = $api->checkSignature($signature, $json);

            if (!$signatureMatches) {
                http_response_code(403);
                die();
            }

            $order = wc_get_order(sanitize_text_field($body->merchant_reference));

            /**
             * Check that Montonio was used to place the order
             */
            if (!$order || $order->get_payment_method() !== $this->id) {
                http_response_code(404);
                die();
            }

            $paymentStatus = $body->status;
        } else {

            if (!isset($_REQUEST['order_id'])) {
                http_response_code(404);
                die();
            }

            $order_id = sanitize_text_field($_REQUEST['order_id']);
            $order    = wc_get_order($order_id);

            /**
             * Check that Montonio was used to place the order
             */
            if (!$order || $order->get_payment_method() !== $this->id) {
                http_response_code(404);
                die();
            }

            /**
             * Check nonce
             */
            if (!wp_verify_nonce( $_REQUEST['_wpnonce'], self::NONCE_HIREPURCHASE_CALLBACK . $order_id)) {
                http_response_code(404);
                die();
            }

            /**
             * Get application for this order from Montonio
             */
            $api = new MontonioAPI(
                $this->getPublicKey(),
                $this->getSecretKey(),
                ($this->getSandbox()) ? 'sandbox' : 'production'
            );

            $res = $api->get_montonio_application($order_id);

            if ('SUCCESS' !== $res['status']) {

                http_response_code(503);
                die('There was a problem with processing the order.');
            }

            $application = $res['data'];

            /**
             * Get validator
             */
            if (!class_exists('HirepurchaseValidator')) {
                require_once 'libraries/HirepurchaseValidator.php';
            }

            try {
                Montonio_Hirepurchase_Validator::checkAmount($application->loan_amount, $order->get_total());
                Montonio_Hirepurchase_Validator::checkCurrency($application->currency_code, $order->get_currency());
            } catch (Exception $e) {

                $order->add_order_note(__('Montonio: Unable to verify integrity of hire purchase application. Payment was not processed. Please contact support.', 'woo-montonio'));
                wc_add_notice('Payment failed. Montonio application was not completed.', 'error');
                http_response_code(400);
                die();
            }

            $paymentStatus = $application->status;
        }

        // *** Authorized ***
        /** @var string $paymentStatus */

        // if not handled for any reason, redirect the user to homepage
        $returnUrl = wc_get_checkout_url();

        switch ($order->get_status()) {
            /**
                 * Order is waiting payment (the most common option)
                 */
            case 'pending':
                if ($paymentStatus === 'signed') {
                    $order->update_status(
                        'processing',
                        __('Montonio Payments: Order has been paid for successfully.', 'woo-montonio')
                    );
                    $returnUrl = $order->get_checkout_order_received_url();
                } else {
                    $order->add_order_note(__('Montonio: Unable to get financing', 'woo-montonio'));
                    wc_add_notice(__('Unable to get financing through Montonio. Please choose another payment method', 'woo-montonio'));
                }
                break;

            /**
                 * Order status has automatically timed out
                 * or manually cancelled from admin
                 */
            case 'cancelled':
                if ($paymentStatus === 'signed') {
                    $order->update_status(
                        'on-hold',
                        __('Montonio Payments: Order has been paid for, but the order either timed out or was cancelled manually. Please confirm the order manually.', 'woo-montonio')
                    );
                    wc_add_notice(__('Order has been paid for, but the order either timed out or was cancelled manually. Please confirm with the merchant that your order will be fulfilled', 'woo-montonio'), 'notice');
                    $returnUrl = $order->get_checkout_order_received_url();
                } else {
                    $order->add_order_note(__('Montonio: Unable to get financing', 'woo-montonio'));
                    wc_add_notice(__('Unable to get financing through Montonio. Please choose another payment method', 'woo-montonio'));
                }
                break;

            /**
                 * Revisiting the URL or coming back via "Return to Merchant" button
                 */
            case 'processing':
            case 'completed':
                $returnUrl = $order->get_checkout_order_received_url();
                break;

            default:
                wc_add_notice(__('Unable to finish the payment. Please try again or choose a different payment method.', 'woo-montonio'), 'notice');
                break;
        }

        if ($is_notification) {
            http_response_code(200);
        } else {
            wp_redirect($returnUrl);
        }

        exit;
    }

    // =========================================================================
    // Iframe Customer Journey
    // =========================================================================

    /**
     * Prepare Iframe customer journey data for given order
     *
     * @param int $order_id
     * @param WC_Abstract_Order
     *
     * @return Montonio_Financing_Request
     *
     */
    protected function prepareIframeJourneyData($order_id, $order)
    {
        if (!class_exists('Montonio_Financing_Request')) {
            require_once 'class-montonio-financing-request.php';
        }

        $iframeRequest = Montonio_Financing_Request::create()
            ->setPublicKey($this->getPublicKey())
            ->setSecretKey($this->getSecretKey())
            ->setSandbox($this->getSandbox())
            ->setOrderID($order_id)
            ->setOrder($order)
            ->setReturnUrl($order->get_checkout_payment_url(true))
            ->prepareCustomerDetails()
            ->prepareProducts();

        return $iframeRequest;
    }

    /**
     * This method is called when user is redirected to Receipt page.
     * Usually logic in this function is made in process_payment(),
     * but since we are using an iframe, it is better to handle logic
     * here.
     */
    public function setup_montonio_form($orderID)
    {

        if ($_REQUEST['access_token']) {

            $sanitizedMontonioDraft = sanitize_text_field($_REQUEST['access_token']);
            if (!class_exists('Montonio_Financing_Iframe')) {
                require_once 'class-montonio-financing-iframe.php';
            }

            $iframe = Montonio_Financing_Iframe::create()
                ->setDraftToken($sanitizedMontonioDraft)
                ->setSandbox($this->getSandbox())
                ->setOrderID($orderID)
                ->setupMontonio();
        }
    }

    public function update_merchant_system()
    {
        try {
            // see if order id is provided
            if (!isset($_REQUEST['order_id'])) {
                throw new Exception('montonio-woocommerce/includes/class-wc-montonio-gateway.php:update_merchant_system:1');
            }

            // check nonce
            check_ajax_referer('montonio_update_url_return', 'security');

            // sanitize order id & see if exists
            $sanitizedOrderId = sanitize_text_field($_REQUEST['order_id']);
            $order            = wc_get_order($sanitizedOrderId);
            if (!$order) {
                throw new Exception('montonio-woocommerce/includes/class-wc-montonio-gateway.php:update_merchant_system:2');
            }

            // start Montonio Validation
            if (!class_exists('Montonio_Financing_Validation')) {
                require_once 'class-montonio-financing-validation.php';
            }

            $returnUrl = Montonio_Financing_Validation::create()
                ->setPublicKey($this->getPublicKey())
                ->setSecretKey($this->getSecretKey())
                ->setSandbox($this->getSandbox())
                ->setOrderID($sanitizedOrderId)
                ->setOrder($order)
                ->setPaymentCompletedStatus($this->getPaymentCompletedStatus())
                ->setPaymentCancelledStatus($this->getPaymentCancelledStatus())
                ->getReturnUrlForApplication();
            exit($this->_getAjaxResponse('OK', $returnUrl));
        } catch (Exception $e) {
            exit($this->_getAjaxResponse('ERROR', 'Unable to process order'));
        }
    }

    private function _getAjaxResponse($status, $data)
    {
        return json_encode(array(
            'status' => $status,
            'data'   => $data,
        ));
    }

    protected function getOrderLogMsg($order, $errorMsg, $sendLog = false)
    {
        $fullLog = $errorMsg . ':'
        . ' Order #' . $order->get_id() . ';'
        . ' Amount: ' . $order->get_total() . $order->get_currency();

        if ($sendLog) {
            error_log($fullLog);
            return $sendLog;
        } else {
            return $fullLog;
        }
    }

    protected function getLocalLang($delimiter)
    {
        $lang = explode($delimiter, get_locale());

        return $lang[0];
    }

    protected function generateTabs($tabs)
    {
        $data = [];
        foreach ($tabs as $key => $value) {
            $data[$key]['name']  = $value['name'];
            $data[$key]['slice'] = $this->generate_settings_html($value['slice'], false);
        }

        return $data;
    }

    protected function updateAdminSettings($data)
    {
        $this->form_fields['paymentCancelledStatus']['options'] = $data['statuses'];
        $this->form_fields['paymentCompletedStatus']['options'] = $data['statuses'];
        $this->form_fields['paymentPendingStatus']['options']   = $data['statuses'];
    }

    protected function updateOrderStatus($order, $status)
    {
        $orderStatusFiltered = str_replace("wc-", "", $status);
        $order->update_status(
            $orderStatusFiltered,
            __('Montonio: Status changed to ', 'woo-montonio') . $orderStatusFiltered . '<br />',
            true
        );
    }

    public function validate_secretKey_field($key, $value)
    {
        if (1 > strlen($value)) {
            WC_Admin_Settings::add_error(esc_html__(
                'Secret key must not be empty',
                'woo-montonio'
            ));
        }

        return $value;
    }

    protected function _resolvePaymentHandleStyle($selectedStyle)
    {
        $styles = array(
            'default.css', // 0
            'nostyle.css', // 1,
            'whitelabel.css', // 2
        );
        if ($selectedStyle < count($styles)) {
            return $styles[$selectedStyle];
        } else {
            return 'default.css';
        }
    }

    /**
     * @return string
     */
    public function getTitle()
    {
        return $this->title;
    }

    /**
     * @param string $title
     */
    public function setTitle($title)
    {
        $this->title = $title;
    }

    /**
     * @return string
     */
    public function getDescription()
    {
        return $this->description;
    }

    /**
     * @param string $description
     */
    public function setDescription($description)
    {
        $this->description = $description;
    }

    /**
     * @return integer
     */
    public function getPublicKey()
    {
        return $this->publicKey;
    }

    /**
     * @param integer $publicKey
     */
    public function setProjectId($publicKey)
    {
        $this->publicKey = $publicKey;
    }

    /**
     * @return string
     */
    public function getSecretKey()
    {
        return $this->secretKey;
    }

    /**
     * @param string $secretKey
     */
    public function setPassword($secretKey)
    {
        $this->secretKey = $secretKey;
    }

    /**
     * @return boolean
     */
    public function getSandbox()
    {
        return $this->sandbox;
    }

    /**
     * @param boolean $sandbox
     */
    public function setSandbox($mode)
    {
        $this->sandbox = $mode;
    }

    /**
     * @return string
     */
    public function getPaymentCancelledStatus()
    {
        return $this->paymentCancelledStatus;
    }

    /**
     * @param string $paymentCancelledStatus
     */
    public function setPaymentCancelledStatus($paymentCancelledStatus)
    {
        $this->paymentCancelledStatus = $paymentCancelledStatus;
    }

    /**
     * @return string
     */
    public function getPaymentCompletedStatus()
    {
        return $this->paymentCompletedStatus;
    }

    /**
     * @param string $paymentCompletedStatus
     */
    public function setPaymentCompletedStatus($paymentCompletedStatus)
    {
        $this->paymentCompletedStatus = $paymentCompletedStatus;
    }

    /**
     * @return string
     */
    public function getPaymentPendingStatus()
    {
        return $this->paymentPendingStatus;
    }

    /**
     * @param string $paymentPendingStatus
     */
    public function setPaymentPendingStatus($paymentPendingStatus)
    {
        $this->paymentPendingStatus = $paymentPendingStatus;
    }

    /**
     * @return string
     */
    public function getPaymentHandleLogo()
    {
        return $this->paymentHandleLogo;
    }

    /**
     * @param string $paymentPendingStatus
     */
    public function setPaymentHandleLogo($paymentHandleLogo)
    {
        $this->paymentHandleLogo = $paymentHandleLogo;
    }

    /**
     * @return string
     */
    public function getShowLogo()
    {
        return $this->showLogo;
    }

    /**
     * @param string $paymentPendingStatus
     */
    public function setShowLogo($showLogo)
    {
        $this->showLogo = $showLogo;
    }

    /**
     * @return object
     */
    public function getPluginSettings()
    {
        return $this->pluginSettings;
    }

    /**
     * @param object $pluginSettings
     */
    public function setPluginSettings($pluginSettings)
    {
        $this->pluginSettings = $pluginSettings;
    }
}
