<?php

defined('ABSPATH') or exit;

/**
 * This class holds logic for the Settings screen located at
 * Woocommerce -> Payments -> Montonio Payments
 *
 * The class handles building the settings form and input validation.
 */
class Montonio_Payments_Settings
{
    const DEFAULT_CURRENCY = 'EUR';
    const DEFAULT_LANG     = 'en';

    /**
     * @var integer
     */
    protected $settings;

    /**
     * @var string
     */
    protected $currency;

    /**
     * @var string
     */
    protected $lang;

    /**
     * @var array
     */
    protected $formFields;

    /**
     * @var array
     */
    protected $calculatorFormFields;

    /**
     * @return self
     */
    static public function create()
    {
        return new self();
    }

    /**
     * Montonio_Financing_Settings constructor.
     */
    public function __construct()
    {
        $this->lang       = $this::DEFAULT_LANG;
        $this->currency   = $this::DEFAULT_CURRENCY;
        $this->formFields = $this->getConfigurationFields();

        /**
         * Setup syncing banks on settings save
         */
        add_filter('woocommerce_settings_api_sanitized_fields_montonio_payments', array($this, 'montonio_validate_settings'));
    }

    /**
     * Perform validation on settings after saving them
     *
     * @param Array $settings - New Admin Settings data
     * @return Array
     */
    public function montonio_validate_settings($settings)
    {
        $syncRequest = true;
        /**
         * Disable the payment gateway if keys are not provided
         */
        if (!$settings['montonioPaymentsAccessKey'] || !$settings['montonioPaymentsSecretKey']) {
            add_action('admin_notices', array($this, 'montonio_error_missing_apikeys'));
            $settings['enabled'] = 'no';
            $syncRequest = false;
        }

        /**
         * sync banks list
         */
        $settings['montonioPaymentsBankList'] = $this->sync_banks($syncRequest, $settings);

        /**
         * sanitize other fields such as CSS
         */
        return $this->montonio_sanitize_fields($settings);
    }

    /**
     * Undocumented function
     *
     * @param Array $settings - New Admin Settings data
     * @return string - the JSON-encoded banklist array
     */
    private function sync_banks($sync = true, $settings)
    {
        if (!class_exists('MontonioPaymentsSDK')) {
            require_once 'libraries/MontonioPaymentsSDK.php';
        }

        /**
         * Get the bearer token we use for the request
         */
        $token = MontonioPaymentsSDK::getBearerToken(
            $settings['montonioPaymentsAccessKey'],
            $settings['montonioPaymentsSecretKey'],
        );

        /**
         * Fetch the banklist from Montonio
         */
        try {

            /**
             * ApiKeys not provided
             */
            if (!$sync) {
                throw new Exception();
            }

            $env    = $settings['montonioPaymentsEnvironment'];
            $apiUrl = $env === 'sandbox'
            ? 'https://api.sandbox-payments.montonio.com/pis/merchants/aspsps'
            : 'https://api.payments.montonio.com/pis/merchants/aspsps';

            $request = wp_remote_request($apiUrl, array(
                'method'  => 'GET',
                'headers' => array(
                    'Authorization' => 'Bearer ' . $token,
                ),
            ));

            /**
             * Rollback if unsuccessful request
             */
            $code = wp_remote_retrieve_response_code($request);
            if ($code !== 200) {
                throw new Exception();
            }

            return wp_remote_retrieve_body($request);
        } catch (Exception $e) {

            /**
             * If there has previously been a successful sync of banks, then keep the banklist
             * otherwise create an empty array for banklist to prevent errors elsewhere
             */
            add_action('admin_notices', array($this, 'montonio_error_failed_to_sync_banks'));
            return $this->settings['montonioPaymentsBankList'] ?? json_encode([]);
        }
    }

    // =========================================================================
    // Admin notice messages
    // =========================================================================

    public function montonio_error_failed_to_sync_banks()
    {
        return $this->montonio_show_notice(
            __('Montonio Payments was unable to sync banks in the background.', 'woo-montonio'),
            'error'
        );
    }

    public function montonio_error_missing_apikeys()
    {
        return $this->montonio_show_notice(
            __('Montonio Payments was disabled due to missing API keys.', 'woo-montonio'),
            'error'
        );
    }

    public function montonio_show_notice($message, $severity = 'success')
    {
        if (!$message) {
            return;
        }

        echo '<div class="notice notice-' . $severity . ' is-dismissible"><p>';
        echo $message;
        echo '</p></div>';
    }

    /**
     * Sanitize Payment Handle CSS using CSSTidy to prevent malicious attacks
     *
     * @param Array $settings
     * @return Array The sanitized settings
     */
    public function montonio_sanitize_fields($settings)
    {
        if (isset($settings['montonioPaymentsPaymentHandleCss'])) {
            include_once 'libraries/csstidy/class.csstidy.php';

            $csstidy = new csstidy();
            $csstidy->set_cfg('remove_bslash', false);
            $csstidy->set_cfg('compress_colors', false);
            $csstidy->set_cfg('compress_font-weight', false);
            $csstidy->set_cfg('discard_invalid_properties', true);
            $csstidy->set_cfg('merge_selectors', false);
            $csstidy->set_cfg('remove_last_;', false);
            $csstidy->set_cfg('css_level', 'CSS3.0');
            $csstidy->parse($settings['montonioPaymentsPaymentHandleCss']);

            $settings['montonioPaymentsPaymentHandleCss'] = $csstidy->print->plain();
        }

        return $settings;
    }

    /**
     * @param object $tabs
     * @param boolean [Optional] $print
     *
     * @return boolean|string
     */
    public function buildAdminFormHtml($tabs, $print = true)
    {
        $htmlData = $this->generateFormFields($tabs);

        $html = '<div class="plugin_config">';
        $html .= '<h2>' . $htmlData['links'] . '</h2>';
        $html .= '<div style="clear:both;"><hr /></div>';
        $html .= $htmlData['tabs'];
        $html .= '</div>';

        if ($print) {
            print_r($html);
            return $print;
        } else {
            return $html;
        }
    }

    /**
     * @param object $tabs
     *
     * @return array
     */
    protected function generateFormFields($tabs)
    {
        $tabsLink    = '';
        $tabsContent = '';
        foreach ($tabs as $key => $value) {
            $tabsLink .= '<a href="javascript:void(0)"';
            $tabsLink .= ' id="tab' . $key . '" class="nav-tab"';
            $tabsLink .= ' data-cont="content' . $key . '">';
            $tabsLink .= $value['name'] . '</a>';

            $tabsContent .= '<div id="content' . $key . '" class="tabContent">';
            $tabsContent .= '<table class="form-table">' . $value['slice'] . '</table>';
            $tabsContent .= '</div>';
        }

        return [
            'links' => $tabsLink,
            'tabs'  => $tabsContent,
        ];
    }

    /**
     * The form fields at Montonio Payments configuration page
     *
     * @return array
     */
    private function getConfigurationFields()
    {
        return array(
            'enabled'                          => array(
                'title'       => __('Enable Montonio Payments', 'woo-montonio'),
                'label'       => __('Enabled', 'woo-montonio'),
                'type'        => 'checkbox',
                'description' => '',
                'default'     => 'no',
            ),
            'montonioPaymentsEnvironment'      => array(
                'type'        => 'select',
                'title'       => __('Environment', 'woo-montonio'),
                'class'       => 'wc-enhanced-select',
                'default'     => 'production',
                'description' => __('Use the Sandbox environment for testing only', 'woo-montonio'),
                'options'     => array(
                    'production' => 'Production',
                    'sandbox'    => 'Sandbox',
                ),
            ),
            'montonioPaymentsAccessKey'        => array(
                'title'   => __('Access Key', 'woo-montonio'),
                'type'    => 'text',
                'default' => __('', 'woo-montonio'),
            ),
            'montonioPaymentsSecretKey'        => array(
                'title'       => __('Secret Key', 'woo-montonio'),
                'type'        => 'password',
                'description' => __('Montonio Access and Secret keys can be obtained from <a target="_blank" href="https://partner.montonio.com">Montonio Partner System</a><br>and Sandbox mode keys from <a target="_blank" href="https://sandbox-partner.montonio.com">Montonio Sandbox Partner System</a>.', 'woo-montonio'),
                'default'     => __('', 'woo-montonio'),
            ),
            'paymentHandleStyle'               => array(
                'title'       => __('Payment handle style', 'woo-montonio'),
                'type'        => 'select',
                'class'       => 'wc-enhanced-select',
                'default'     => 'list_logos',
                'description' => __('This controls how to display Montonio Payments at checkout', 'woo-montonio'),
                'options'     => array(
                    'list_logos' => 'Show logos in list',
                    'list_names' => 'Show names in list',
                    'grid_logos' => 'Show logos in grid',
                ),
            ),
            'montonioPaymentsPaymentHandleCss' => array(
                'title'       => __('CSS for checkout', 'woo-montonio'),
                'type'        => 'textarea',
                'css'         => 'width: 400px;',
                'description' => __(
                    'Here you can insert additional CSS rules for checkout.',
                    'woo-montonio'
                ),
            ),
        );
    }

    /**
     * @return integer
     */
    public function getSettings()
    {
        return $this->settings;
    }

    /**
     * @param integer $settings
     *
     * @return self
     */
    public function setSettings($settings)
    {
        $this->settings = $settings;
        return $this;
    }

    /**
     * @return string
     */
    public function getCurrency()
    {
        return $this->currency;
    }

    /**
     * @param string $currency
     *
     * @return self
     */
    public function setCurrency($currency)
    {
        $this->currency = $currency;
        return $this;
    }

    /**
     * @return string
     */
    public function getLang()
    {
        return $this->lang;
    }

    /**
     * @param string $lang
     *
     * @return self
     */
    public function setLang($lang)
    {
        $this->lang = $lang;

        return $this;
    }

    /**
     * @return array
     */
    public function getFormFields()
    {
        return $this->formFields;
    }

    /**
     * @param array $formFields
     *
     * @return self
     */
    public function setFormFields($formFields)
    {
        $this->formFields = $formFields;

        return $this;
    }
}
