<?php
defined('ABSPATH') or exit;

class Montonio_Payments extends WC_Payment_Gateway
{

    /**
     * @var object
     */
    protected $pluginSettings;

    public function __construct()
    {
        $this->id                 = 'montonio_payments';
        $this->icon               = 'https://public.montonio.com/logo/montonio_dark_icononly.png';
        $this->title              = __('Maksa pangas', 'woo-montonio');
        $this->has_fields         = false;
        $this->method_title       = __('Montonio Payments', 'woo-montonio');
        $this->method_description = __('Allows bank payments via Montonio Payment Initiation Service', 'woo-montonio');
        $this->init_form_fields();
        $this->init_settings();

        /**
         * Register the callback function for when returning from Montonio
         */
        add_action('woocommerce_api_montonio_payments', array($this, 'check_payment_response'));

        /**
         * Register payment notification webhook handler
         */
        add_action('woocommerce_api_montonio_payments_notification', array($this, 'check_payment_notification'));

        /**
         * Show Montonio settings in Woocommerce->settings->payments
         */
        add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));

        /**
         * Add CSS class to Montonio Logo at checkout
         */
        add_filter('woocommerce_gateway_icon', array($this, 'add_montonio_logo_callback'), 10, 3);
    }

    /**
     * Setup how Montonio Payments looks at checkout
     *
     * @return string - the description HTML
     */
    public function get_description()
    {
        if (!class_exists('Montonio_Payments_Checkout')) {
            require_once 'class-montonio-payments-checkout.php';
        }

        wp_enqueue_style(
            'montonio-payments-payment-handle-logo',
            MONTONIO_PLUGIN_URL . 'payments/assets/css/payment-handle/payment-handle.css'
        );

        $checkout = new Montonio_Payments_Checkout();
        $checkout->set_payment_handle_style($this->get_option('paymentHandleStyle'));
        $checkout->set_payment_handle_extra_css($this->get_option('montonioPaymentsPaymentHandleCss'));
        $checkout->set_banklist($this->get_option('montonioPaymentsBankList'));
        $description = $checkout->get_description_html();

        $checkout->register_payment_handle_style();
        $checkout->register_payment_handle_script();

        return $description;
    }

    // =========================================================================
    // Checkout style
    // =========================================================================
    public function register_checkout_style()
    {
        $this->title = __('Maksa pangas', 'woo-montonio');
        $this->icon  = 'https://public.montonio.com/logo/montonio_dark_icononly.png';
    }

    public function add_montonio_logo_callback($icon, $id)
    {
        if ($id == $this->id) {
            return str_replace('src="', 'id="montonio-payments-checkout-logo" src="', $icon);
        }
        return $icon;
    }

    // =========================================================================
    // Plugin settings and setup
    // =========================================================================

    public function init_form_fields()
    {
        if (!class_exists('Montonio_Payments_Settings')) {
            require_once 'class-montonio-payments-settings.php';
        }
        $this->setPluginSettings(Montonio_Payments_Settings::create());
        $this->form_fields = $this->getPluginSettings()->getFormFields();
    }

    public function init_settings()
    {
        parent::init_settings();

        /**
         * Save current settings to the Montonio_Payments_Settings instance
         */ 
        $settingsClass = $this->getPluginSettings();
        $settingsClass->setSettings($this->settings);
    }

    // =========================================================================
    // Payment Flow: Redirect and Callback
    // =========================================================================

    /**
     * Process Payment.
     *
     * @param int $order_id Order ID.
     * @return array
     */
    public function process_payment($order_id)
    {
        if (!class_exists('MontonioPaymentsSDK')) {
            require_once 'libraries/MontonioPaymentsSDK.php';
        }

        /**
         * Get the newly created order and prepare it for submission
         */
        $order = wc_get_order($order_id);
        $order->add_order_note(__('Montonio Payments: Checkout process is started', 'woo-montonio'));

        /**
         * Create new Montonio Payments SDK instance
         */
        $montonioPayments = new MontonioPaymentsSDK(
            $this->get_option('montonioPaymentsAccessKey'),
            $this->get_option('montonioPaymentsSecretKey'),
            $this->get_option('montonioPaymentsEnvironment')
        );

        /**
         * Prepare Payment Data for Montonio Payments
         */
        $paymentData = array(
            'amount'                    => $order->get_total(),
            'currency'                  => $order->get_currency(),
            'merchant_reference'        => $order_id,
            'checkout_email'            => (string) $order->get_billing_email(),
            'checkout_first_name'       => (string) $order->get_billing_first_name(),
            'checkout_last_name'        => (string) $order->get_billing_last_name(),
            'checkout_phone_number'     => (string) $order->get_billing_phone(),
            'merchant_notification_url' => str_replace(
                'https:', 'http:',
                add_query_arg('wc-api', 'montonio_payments_notification', home_url('/'))

            ),
            'merchant_return_url'       => str_replace(
                'https:', 'http:',
                add_query_arg('wc-api', 'Montonio_Payments', home_url('/'))
            ),
        );

        /**
         * Add preselected ASPSP to query if provided
         */
        if ($_POST && $_POST['montonio_payments_preselected_aspsp']) {
            $paymentData['preselected_aspsp'] = $_POST['montonio_payments_preselected_aspsp'];
        }

        if (in_array(get_locale(), array('en_US', 'lt', 'et'))) {
            $paymentData['preselected_locale'] = get_locale();
        }

        $montonioPayments->setPaymentData($paymentData);

        $paymentUrl = $montonioPayments->getPaymentUrl();

        /**
         * Return response after which redirect to Montonio Payments will happen
         */
        return array(
            'result'   => 'success',
            'redirect' => $paymentUrl,
        );
    }

    public function check_payment_notification()
    {
        return $this->check_payment_response(true);
    }

    /**
     * Check callback from Montonio
     * and redirect user: thankyou page for success, checkout on declined/failure
     *
     * @param bool $is_notification True if webhook notification, false otherwise
     *
     * @return void
     */
    public function check_payment_response($is_notification = false)
    {

        /**
         * Get Payment Token from request
         */
        if (!isset($_REQUEST['payment_token'])) {
            wc_add_notice(__('Unable to finish the payment. Please try again or choose a different payment method.', 'woo-montonio'), 'notice');
            wp_redirect(wc_get_checkout_url());
            exit;
        }

        $token = sanitize_text_field($_REQUEST['payment_token']);

        /**
         * Decode and Validate Payment Token with Montonio Payments SDK
         */
        if (!class_exists('MontonioPaymentsSDK')) {
            require_once 'libraries/MontonioPaymentsSDK.php';
        }

        $response = MontonioPaymentsSDK::decodePaymentToken($token, $this->get_option('montonioPaymentsSecretKey'));

        $paymentStatus = sanitize_text_field($response->status);

        /**
         * Get order
         */
        $order = wc_get_order(sanitize_text_field($response->merchant_reference));

        /**
         * Check that Montonio was used to place the order
         */
        if (!$order || $order->get_payment_method() !== $this->id) {
            http_response_code(404);
            die();
        }

        /**
         * Return URL if payment for order failed
         */
        $returnUrl = wc_get_checkout_url();

        switch ($order->get_status()) {

            /**
                 * Order is waiting payment (the most common option)
                 */
            case 'pending':
                if ($paymentStatus === 'finalized') {
                    $order->update_status(
                        'processing',
                        __('Montonio Payments: Order has been paid for successfully.', 'woo-montonio')
                    );
                    $returnUrl = WC_Payment_Gateway::get_return_url($order);
                }
                break;

            /**
                 * Order status has automatically timed out
                 * or manually cancelled from admin
                 */
            case 'cancelled':
                if ($paymentStatus === 'finalized') {
                    $order->update_status(
                        'on-hold',
                        __('Montonio Payments: Order has been paid for, but the order either timed out or was cancelled manually. Please confirm the order manually.', 'woo-montonio')
                    );
                    wc_add_notice(__('Order has been paid for, but the order either timed out or was cancelled manually. Please confirm with the merchant that your order will be fulfilled', 'woo-montonio'), 'notice');
                    $returnUrl = WC_Payment_Gateway::get_return_url($order);
                }
                break;

            /**
                 * Revisiting the URL or coming back via "Return to Merchant" button
                 */
            case 'processing':
            case 'completed':
                $returnUrl = WC_Payment_Gateway::get_return_url($order);
                break;

            default:
                wc_add_notice(__('Unable to finish the payment. Please try again or choose a different payment method.', 'woo-montonio'), 'notice');
                break;
        }

        if ($is_notification) {
            http_response_code(200);
        } else {
            wp_redirect($returnUrl);
        }

        exit;
    }

    // =========================================================================
    // Getters and Setters
    // =========================================================================

    /**
     * @return object
     */
    public function getPluginSettings()
    {
        return $this->pluginSettings;
    }

    /**
     * @param object $pluginSettings
     */
    public function setPluginSettings($pluginSettings)
    {
        $this->pluginSettings = $pluginSettings;
    }
}
