<?php

/**
 * We use php-jwt for JWT creation
 */
require_once 'jwt/JWT.php';

/**
 * Process order data after place order clicked
 */
class MontonioPaymentsSDK
{
    /**
     * Payment Data for Montonio Payment Token generation
     *
     * @var array
     */
    protected $_paymentData;

    protected $_accessKey;
    protected $_secretKey;
    protected $_environment;

    const MONTONIO_PAYMENTS_SANDBOX_APPLICATION_URL = 'https://sandbox-payments.montonio.com';

    const MONTONIO_PAYMENTS_APPLICATION_URL = 'https://payments.montonio.com';

    public function __construct($accessKey, $secretKey, $environment)
    {
        $this->_accessKey   = $accessKey;
        $this->_secretKey   = $secretKey;
        $this->_environment = $environment;
    }

    /**
     * Get the URL string where to redirect the customer to
     *
     * @return string
     */
    public function getPaymentUrl()
    {
        $base = ($this->_environment === 'sandbox')
        ? self::MONTONIO_PAYMENTS_SANDBOX_APPLICATION_URL
        : self::MONTONIO_PAYMENTS_APPLICATION_URL;

        return $base . '?payment_token=' . $this->_generatePaymentToken();
    }

    /**
     * Generate JWT from Payment Data
     *
     * @return void
     */
    protected function _generatePaymentToken()
    {
        /**
         * Parse Payment Data to correct data types
         * and add additional data
         */
        $paymentData = array(
            'amount'                => (float) $this->_paymentData['amount'],
            'currency'              => (string) $this->_paymentData['currency'],
            'access_key'            => (string) $this->_accessKey,
            'merchant_reference'    => (string) $this->_paymentData['merchant_reference'],
            'merchant_return_url'   => (string) $this->_paymentData['merchant_return_url'],
            'checkout_email'        => (string) $this->_paymentData['checkout_email'],
            'checkout_first_name'   => (string) $this->_paymentData['checkout_first_name'],
            'checkout_last_name'    => (string) $this->_paymentData['checkout_last_name'],
            'checkout_phone_number' => (string) $this->_paymentData['checkout_phone_number'],
        );

        if (isset($this->_paymentData['merchant_notification_url'])) {
            $paymentData['merchant_notification_url'] = (string) $this->_paymentData['merchant_notification_url'];
        }

        if (isset($this->_paymentData['preselected_aspsp'])) {
            $paymentData['preselected_aspsp'] = (string) $this->_paymentData['preselected_aspsp'];
        }

        if (isset($this->_paymentData['preselected_locale'])) {
            $paymentData['preselected_locale'] = (string) $this->_paymentData['preselected_locale'];
        }

        // add expiry to payment data for JWT validation
        $exp = time() + (10 * 60);
        $paymentData['exp'] = $exp;

        return Firebase\JWT\JWT::encode($paymentData, $this->_secretKey);
    }

    /**
     * Set payment data
     *
     * @param array $paymentData
     * @return MontonioPaymentsSDK
     */
    public function setPaymentData($paymentData)
    {
        $this->_paymentData = $paymentData;
        return $this;
    }

    public static function decodePaymentToken($token, $secretKey)
    {
        return Firebase\JWT\JWT::decode($token, $secretKey, array('HS256'));
    }

    public static function getBearerToken($accessKey, $secretKey) {
        $data = array(
            'access_key' => $accessKey
        );
        
        return Firebase\JWT\JWT::encode($data, $secretKey);
    }
}
